@@ -1,10 +1,12 @@
 #define G_CPCM_BASE 			"/apps/maemo/cpcm/"
 #define G_CPCM_MACHINE			G_CPCM_BASE "machine"
+#define G_CPCM_MONOCHROME		G_CPCM_BASE "monochrome"
 #define G_CPCM_DRIVE_A 			G_CPCM_BASE "drivea"
 #define G_CPCM_DRIVE_B 			G_CPCM_BASE "driveb"
 #define G_CPCM_SOUND 			G_CPCM_BASE "sound"
 #define G_CPCM_DISP_FRAMERATE	G_CPCM_BASE "display-framerate"
 #define G_CPCM_SAVEDIR			G_CPCM_BASE "savedir"
+#define G_CPCM_ROMDIR			G_CPCM_BASE "romdir"
 #define G_CPCM_SCREENSAVER		G_CPCM_BASE "saver"
 #define G_CPCM_JOY1_KEY			G_CPCM_BASE "joy1/keyboard/"
 #define G_CPCM_JOY2_KEY			G_CPCM_BASE "joy2/keyboard/"
