@@ -26,6 +26,8 @@
 #include <fcntl.h>
 
 #include <SDL/SDL.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "cap32.h"
 #include "z80.h"
@@ -755,9 +757,9 @@ psp_update_keys(void)
   return 0;
 }
 
+extern SDL_Rect exit_rect;
 static byte maemo_key_matrix[16];
-static byte maemo_keys[0x200];
-static byte maemo_keys_mod[0x200];
+static byte maemo_keys[0x1000];
 static byte maemo_joy_keys[2];
 static byte maemo_joy_ts[2];
 static byte maemo_joy_acc[2];
@@ -766,12 +768,27 @@ static byte maemo_joy_map[2][6] = {
 		{	0x65,	0x64,	0x60,	0x61,	0x62,	0x63	}
 };
 
-inline static void set_key( unsigned char key ) {
-	maemo_key_matrix[ key >> 4 ] &= ~( 1 << ( key & 0x0F ) );
+
+inline static void set_key( SDL_keysym code ) {
+	int keyval;
+	gdk_keymap_translate_keyboard_state( NULL, code.scancode, code.mod & KMOD_MODE ? GDK_MODIFIER_MASK : 0, 0, &keyval, NULL, NULL, NULL );
+	unsigned char key = maemo_keys[ keyval & 0xFFF ];
+	if( key == 0x97 && code.mod & KMOD_MODE ) key = 0x82; //Generate ESC
+	else if( key == 0x22 && code .mod & KMOD_MODE ) key = 0x84; //Generate TAB
+	maemo_key_matrix[ key >> 4 ] &= ~( 1 << ( key & 0x07 ) );
+	if( key != 0xFF && key & 0x08 )
+		maemo_key_matrix[ 0x02 ] &= 0xDF;
 }
 
-inline static void unset_key( unsigned char key ) {
-	maemo_key_matrix[ key >> 4 ] |= 1 << ( key & 0x0F );
+inline static void unset_key( SDL_keysym code ) {
+	int keyval;
+	gdk_keymap_translate_keyboard_state( NULL, code.scancode, code.mod & KMOD_MODE ? GDK_MODIFIER_MASK : 0, 0, &keyval, NULL, NULL, NULL );
+	unsigned char key = maemo_keys[ keyval & 0xFFF ];
+	maemo_key_matrix[ key >> 4 ] |= 1 << ( key & 0x07 );
+	if( key == 0x97 ) maemo_key_matrix[ 0x08 ] |= 0x04; //Release ESC
+	else if( key == 0x22 ) maemo_key_matrix[ 0x08 ] |= 0x10; //Release TAB
+	if( key != 0xFF && key & 0x08 )
+		maemo_key_matrix[ 0x02 ] |= 0x20;
 }
 
 inline static void set_joy( int code ) {
@@ -796,8 +813,8 @@ inline static void update_joy_ts( int x, int y, int mode ) {
 		if( y >= 0 && y <= 160 ) ts |= CPCM_JOY_UP;
 		if( y >= 320 && y <= 480 ) ts |= CPCM_JOY_DOWN;
 	} else if( x >= 700 && x <= 800 && y >= 0 && y <= 480) {
-		if( y <= 160 ) CPC.cpc_exit = 1;
-		else if( y <= 320 ) ts |= CPCM_JOY_F1;
+		if( y <= exit_rect.h && x >= exit_rect.x ) CPC.cpc_exit = 1;
+		else if( y >= 160 && y <= 320 ) ts |= CPCM_JOY_F1;
 		else ts |= CPCM_JOY_F2;
 	}
 	if( CPC.joy[0].ts_enable ) maemo_joy_ts[0] = ts;
@@ -839,15 +856,16 @@ inline static void update_input() {
 }
 
 void maemo_init_keys() {
+	gdk_init(NULL,NULL);
 	memset( maemo_key_matrix, -1, sizeof(maemo_keys) );
 	memset( maemo_keys, -1, sizeof(maemo_keys) );
-	memset( maemo_keys_mod, -1, sizeof(maemo_keys_mod) );
+	//memset( maemo_keys_mod, -1, sizeof(maemo_keys_mod) );
 	memset( maemo_joy_keys, 0, sizeof(maemo_joy_keys) );
 	memset( maemo_joy_ts, 0, sizeof(maemo_joy_ts) );
 	memset( maemo_joy_acc, 0, sizeof(maemo_joy_acc) );
 
-#define XK( a, b ) { maemo_keys[a] = b; }
-#define XKA( a, b, c ) { maemo_keys[a] = b; maemo_keys_mod[a] = c; }
+#define XK( a, b ) { maemo_keys[ a & 0xFFF ] = b; }
+#define XKA( a, b, c ) { maemo_keys[ a & 0xFFF ] = b; } // maemo_keys_mod[a] = c; }
 #define XJ( a, b )
 #include "keyboard.h"
 #undef XJ
@@ -858,6 +876,7 @@ void maemo_init_keys() {
 int maemo_update_keys() {
 #define set( key )
 	SDL_Event event;
+	int keyval;
 	while( SDL_PollEvent(&event) ) {
 		switch( event.type ) {
 		case SDL_KEYDOWN:
@@ -865,17 +884,12 @@ int maemo_update_keys() {
 				CPC.cpc_exit = 1;
 				return 0;
 			}
-			if( event.key.keysym.mod & KMOD_MODE ) {
-				set_key( maemo_keys_mod[ event.key.keysym.sym ] );
-			}
-			else {
-				set_key( maemo_keys[ event.key.keysym.sym ] );
-			}
+			set_key( event.key.keysym );
 			set_joy( event.key.keysym.scancode );
 			break;
 		case SDL_KEYUP:
-			unset_key( maemo_keys_mod[ event.key.keysym.sym ] );
-			unset_key( maemo_keys[ event.key.keysym.sym ] );
+			gdk_keymap_translate_keyboard_state( NULL, event.key.keysym.scancode, event.key.keysym.mod & KMOD_MODE ? GDK_MODIFIER_MASK : 0, 0, &keyval, NULL, NULL, NULL );
+			unset_key( event.key.keysym );
 			unset_joy( event.key.keysym.scancode );
 			break;
 		case SDL_MOUSEBUTTONDOWN:
